@@ -133,7 +133,7 @@ public final class ReaperApplication extends Application<ReaperApplicationConfig
         bootstrap.getConfigurationSourceProvider(), new EnvironmentVariableSubstitutor(false));
     bootstrap.setConfigurationSourceProvider(envSourceProvider);
 
-    bootstrap.addBundle(
+    /*bootstrap.addBundle(
         new ShiroBundle<ReaperApplicationConfiguration>() {
           @Override
           public void run(ReaperApplicationConfiguration configuration, Environment environment) {
@@ -146,7 +146,7 @@ public final class ReaperApplication extends Application<ReaperApplicationConfig
           protected ShiroConfiguration narrow(ReaperApplicationConfiguration configuration) {
             return configuration.getAccessControl().getShiroConfiguration();
           }
-        });
+        });*/
 
     bootstrap.setConfigurationSourceProvider(
         new SubstitutingSourceProvider(
