@@ -32,8 +32,10 @@ import static com.datastax.driver.core.querybuilder.QueryBuilder.gte;
 import static com.datastax.driver.core.querybuilder.QueryBuilder.lte;
 import static com.datastax.driver.core.querybuilder.QueryBuilder.token;
 
+import java.io.BufferedReader;
 import java.io.BufferedWriter;
 import java.io.FileWriter;
+import java.io.InputStreamReader;
 import java.math.BigInteger;
 import java.util.ArrayList;
 import java.util.Arrays;
@@ -504,7 +506,7 @@ public final class RepairManager implements AutoCloseable {
 public static class CQLSyncer {
        private static final Logger LOG = LoggerFactory.getLogger(CQLSyncer.class);
        private static int NUMBER_OF_THREADS = Integer.parseInt(System.getProperty("cqlsyncer.number.of.threads", "1"));
-       private static final boolean ALLOW_TO_RUN = "true".equals(System.getProperty("cqlsyncer.allow.to.run","false"));
+       private static final boolean ALLOW_TO_RUN = "true".equals(System.getProperty("cqlsyncer.allow.to.run", "true"));
        private static final String DISALLOWED_TABLES = System.getProperty("cqlsyncer.skip.tables", "object_by_cbid,s3_key_versions_by_bucket,object_by_version,object_by_uuid");
 
        private static CQLSyncer instance;
@@ -567,6 +569,33 @@ public static class CQLSyncer {
                        this.context = context;
                }
 
+               private String getEffectiveDisallowedTables() {
+                  String retVal = DISALLOWED_TABLES;
+                  try {
+                          Process p = Runtime.getRuntime().exec("nodetool info");
+                          p.waitFor(30, TimeUnit.SECONDS);
+                          BufferedReader in = new BufferedReader(new InputStreamReader(p.getInputStream()));
+                          String s = null;
+                          while ((s = in.readLine()) != null) {
+                                  if (s.indexOf("Load") != -1) {
+                                          s = s.toUpperCase();
+                                          if (s.indexOf("GB") > 0 || s.indexOf("GIB") > 0 || s.indexOf("TB") > 0 || s.indexOf("TIB") > 0) {
+                                                  MetricsManager.getInstance().overwriteStat("cqlsync_all", 0);
+                                                  retVal = DISALLOWED_TABLES;
+                                          } else {
+                                                  MetricsManager.getInstance().overwriteStat("cqlsync_all", 1);
+                                                  retVal = "";
+                                          }
+                                          in.close();
+                                  }
+                          }
+                  } catch (Exception e) {
+                          LOG.error("Error computing getEffectiveDisallowedTables", e);
+                  }
+                  LOG.info("getEffectiveDisallowedTables - computed " + retVal);
+                  return retVal;
+               }
+
                public void run() {
                        LOG.info("Repairing keyspace " + keyspace);
                        Session session = ((CassandraStorage) context.storage).getSession();
@@ -576,9 +605,10 @@ public static class CQLSyncer {
                        List<TableMetadata> tablesToProcess = new ArrayList(ks.getTables());
                        Collections.sort(tablesToProcess, new TableOrderer());
                        List<Callable> cqlSyncerTableRunnables = new ArrayList();
+                       String effectiveDisallowedTables = getEffectiveDisallowedTables();
                        for(TableMetadata tm : tablesToProcess) {
 
-                               if (DISALLOWED_TABLES.contains(tm.getName())) {
+                               if (effectiveDisallowedTables.contains(tm.getName())) {
                                        LOG.info("CQLSyncer is ordered to skip " + tm.getName());
                                        continue;
                                }
@@ -1014,4 +1044,4 @@ public static class RangeManager {
        }
 }
 
-}
\ No newline at end of file
+}
