@@ -22,6 +22,7 @@ import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.Collection;
 import java.util.Collections;
+import java.util.HashMap;
 import java.util.List;
 import java.util.Map;
 import java.util.Optional;
@@ -74,6 +75,8 @@ final class RepairRunner implements Runnable {
   private float segmentsDone;
   private float segmentsTotal;
   private final List<RingRange> localEndpointRanges;
+  public static HashMap<UUID, Integer> repairRunsWithChangedTopologyCounts = new HashMap();
+  private static Integer endpointCount;
 
   private RepairRunner(
       AppContext context,
@@ -300,11 +303,25 @@ final class RepairRunner implements Runnable {
    */
   private void updateClusterNodeList() throws ReaperException {
          try {
+           if (endpointCount == null) {
+                  endpointCount = new Integer(clusterFacade.getEndpointToHostId(cluster).size());
+           }
            Set<String> liveNodes  = ImmutableSet.copyOf(clusterFacade.getLiveNodes(cluster));
+
            Cluster cluster = context.storage.getCluster(clusterName);
            if (!cluster.getSeedHosts().equals(liveNodes) && !liveNodes.isEmpty()) {
+                  LOG.info("Updating the live node list for cluster {} as it changed since the last repair.", clusterName);
+                  Integer newEndpointCount = new Integer(clusterFacade.getEndpointToHostId(cluster).size());
+                  if (endpointCount != null && newEndpointCount != null && !endpointCount.equals(newEndpointCount)) {
+                          endpointCount = newEndpointCount;
+                   Integer seenAlready = repairRunsWithChangedTopologyCounts.get(repairRunId);
+                   if (seenAlready == null) {
+                        seenAlready = new Integer(0);
+                   }
+                   repairRunsWithChangedTopologyCounts.put(repairRunId, seenAlready+1);
+                  }
              // Updating storage only if the seed lists has changed
-             LOG.info("Updating the seed list for cluster {} as topology changed since the last repair.", clusterName);
+
              context.storage.updateCluster(cluster.with().withSeedHosts(liveNodes).build());
            }
          } catch (Exception e) {
