@@ -67,6 +67,7 @@ import io.cassandrareaper.ReaperApplicationConfiguration.DatacenterAvailability;
 import io.cassandrareaper.ReaperException;
 import io.cassandrareaper.core.Cluster;
 import io.cassandrareaper.core.NodeMetrics;
+import io.cassandrareaper.core.RepairRun;
 import io.cassandrareaper.core.RepairSegment;
 import io.cassandrareaper.core.RepairUnit;
 import io.cassandrareaper.jmx.ClusterFacade;
@@ -116,6 +117,7 @@ public final class SegmentRunner implements RepairStatusHandler, Runnable {
   private final AtomicBoolean releasedSegmentRunner = new AtomicBoolean(false);
   private static UUID lastUsedLockId = null;
   private static SegmentDurationHistoryRecorder recorder = new SegmentDurationHistoryRecorder();
+  private static HashMap<RepairUnit, Long> failingSegmentsForRepair = new HashMap();
 
 
   private SegmentRunner(
@@ -207,6 +209,28 @@ public final class SegmentRunner implements RepairStatusHandler, Runnable {
     int failCountIncrement = 0;
     if (withFailCount) {
        failCountIncrement = 1;
+       Long previousTime = failingSegmentsForRepair.get(repairUnit);
+       if (previousTime == null) {
+          failingSegmentsForRepair.put(repairUnit, System.currentTimeMillis());
+       } else {
+          // lets see how long its been
+          long duration = System.currentTimeMillis() - previousTime;
+          long target = Long.parseLong(System.getProperty("reaper.topology.change.failure.abort.threshold.ms", "7200000"));
+          long necessaryCount = Long.parseLong(System.getProperty("\"reaper.topology.change.failure.abort.threshold.change.notices", "0"));
+          LOG.info("Segment failed.  Considering topology based restart with parameters of " + duration + " " + target + " " +
+                          necessaryCount + " " + RepairRunner.repairRunsWithChangedTopologyCounts.get(segment.getRunId()));
+          if (duration > target && RepairRunner.repairRunsWithChangedTopologyCounts.get(segment.getRunId()) > necessaryCount) {
+                  LOG.info("Decided upon topology based restart of repair");
+                  RepairRun run = context.storage.getRepairRun(segment.getRunId()).get();
+                  run = run.with().lastEvent("Aborted based on topology changes").build(run.getId());
+                  try {
+                          context.repairManager.abortRepairRun(run);
+                  } catch (Exception e) {
+                          e.printStackTrace();
+                  }
+
+          }
+       }
     }
     try {
       context.storage.updateRepairSegment(
@@ -1049,6 +1073,7 @@ public final class SegmentRunner implements RepairStatusHandler, Runnable {
                     .withEndTime(DateTime.now())
                     .withId(segmentId)
                     .build());
+            failingSegmentsForRepair.remove(repairUnit);
 
             successOrFailedNotified.set(true);
             // Since we can get out of order notifications,
