@@ -267,10 +267,10 @@ public final class CassandraStorage implements IStorage, IDistributedStorage {
               .filter((reaperInstanceHost) -> !AppContext.REAPER_INSTANCE_ADDRESS.equals(reaperInstanceHost))
               .collect(Collectors.toList());
 
-          Preconditions.checkState(
-              otherRunningReapers.isEmpty(),
-              "Database migration can not happen with other reaper instances running. Found ",
-              StringUtils.join(otherRunningReapers));
+          //Preconditions.checkState(
+          //    otherRunningReapers.isEmpty(),
+          //    "Database migration can not happen with other reaper instances running. Found ",
+          //    StringUtils.join(otherRunningReapers));
         }
 
         // We now only support migrations starting at version 15 (Reaper 1.2.2)
@@ -376,28 +376,28 @@ public final class CassandraStorage implements IStorage, IDistributedStorage {
             "INSERT INTO repair_run"
                 + "(id,segment_id,repair_unit_id,start_token,end_token,segment_state,fail_count, token_ranges)"
                 + " VALUES(?, ?, ?, ?, ?, ?, ?, ?)")
-        .setConsistencyLevel(ConsistencyLevel.LOCAL_QUORUM);
+        .setConsistencyLevel(ConsistencyLevel.QUORUM);
     insertRepairSegmentIncrementalPrepStmt = session
         .prepare(
             "INSERT INTO repair_run"
                 + "(id,segment_id,repair_unit_id,start_token,end_token,segment_state,coordinator_host,fail_count)"
                 + " VALUES(?, ?, ?, ?, ?, ?, ?, ?)")
-        .setConsistencyLevel(ConsistencyLevel.LOCAL_QUORUM);
+        .setConsistencyLevel(ConsistencyLevel.QUORUM);
     updateRepairSegmentPrepStmt = session
         .prepare(
             "INSERT INTO repair_run"
                 + "(id,segment_id,segment_state,coordinator_host,segment_start_time,fail_count)"
                 + " VALUES(?, ?, ?, ?, ?, ?)")
-        .setConsistencyLevel(ConsistencyLevel.LOCAL_QUORUM);
+        .setConsistencyLevel(ConsistencyLevel.QUORUM);
     insertRepairSegmentEndTimePrepStmt = session
         .prepare("INSERT INTO repair_run(id, segment_id, segment_end_time) VALUES(?, ?, ?)")
-        .setConsistencyLevel(ConsistencyLevel.LOCAL_QUORUM);
+        .setConsistencyLevel(ConsistencyLevel.QUORUM);
     getRepairSegmentPrepStmt = session
             .prepare(
                 "SELECT id,repair_unit_id,segment_id,start_token,end_token,segment_state,coordinator_host,"
                     + "segment_start_time,segment_end_time,fail_count, token_ranges"
                     + " FROM repair_run WHERE id = ? and segment_id = ?")
-            .setConsistencyLevel(ConsistencyLevel.LOCAL_QUORUM);
+            .setConsistencyLevel(ConsistencyLevel.QUORUM);
     getRepairSegmentsByRunIdPrepStmt = session.prepare(
         "SELECT id,repair_unit_id,segment_id,start_token,end_token,segment_state,coordinator_host,segment_start_time,"
             + "segment_end_time,fail_count, token_ranges FROM repair_run WHERE id = ?");
@@ -1208,7 +1208,7 @@ public final class CassandraStorage implements IStorage, IDistributedStorage {
         : getRepairSegmentsByRunIdPrepStmt.bind(runId);
 
     if (State.STARTED == segmentState) {
-      statement = statement.setConsistencyLevel(ConsistencyLevel.LOCAL_QUORUM);
+      statement = statement.setConsistencyLevel(ConsistencyLevel.QUORUM);
     }
     for (Row segmentRow : session.execute(statement)) {
       if (segmentRow.getInt("segment_state") == segmentState.ordinal()) {
@@ -2039,3 +2039,4 @@ public final class CassandraStorage implements IStorage, IDistributedStorage {
   public void purgeNodeOperations() {}
 
 }
+
