@@ -1,6 +1,5 @@
--- vim: st=4 sts=4 sw=4 et:
+-- vim: ts=4 sts=4 sw=4 et:
 
-local cjson      = require "cjson.safe"
 local new_tab    = require "table.new"
 local lrucache   = require "resty.lrucache"
 local resty_lock = require "resty.lock"
@@ -20,6 +19,14 @@ do
         }
     end
 end
+local codec
+do
+    local pok
+    pok, codec = pcall(require, "string.buffer")
+    if not pok then
+        codec = require "cjson"
+    end
+end
 
 
 local now          = ngx.now
@@ -29,11 +36,14 @@ local fmt          = string.format
 local sub          = string.sub
 local find         = string.find
 local type         = type
+local pcall        = pcall
 local xpcall       = xpcall
 local traceback    = debug.traceback
 local error        = error
 local tostring     = tostring
 local tonumber     = tonumber
+local encode       = codec.encode
+local decode       = codec.decode
 local thread_spawn = ngx.thread.spawn
 local thread_wait  = ngx.thread.wait
 local setmetatable = setmetatable
@@ -85,12 +95,12 @@ local marshallers = {
     end,
 
     [4] = function(t)      -- table
-        local json, err = cjson.encode(t)
-        if not json then
-            return nil, "could not encode table value: " .. err
+        local pok, str = pcall(encode, t)
+        if not pok then
+            return nil, "could not encode table value: " .. str
         end
 
-        return json
+        return str
     end,
 }
 
@@ -127,9 +137,9 @@ local unmarshallers = {
     end,
 
     [4] = function(str) -- table
-        local t, err = cjson.decode(str)
-        if not t then
-            return nil, "could not decode table value: " .. err
+        local pok, t = pcall(decode, str)
+        if not pok then
+            return nil, "could not decode table value: " .. t
         end
 
         return t
@@ -164,7 +174,7 @@ end
 
 
 local _M     = {
-    _VERSION = "2.4.0",
+    _VERSION = "2.6.0",
     _AUTHOR  = "Thibault Charbonnier",
     _LICENSE = "MIT",
     _URL     = "https://github.com/thibaultcha/lua-resty-mlcache",
@@ -821,8 +831,8 @@ function _M:get(key, opts, cb, ...)
         error("key must be a string", 2)
     end
 
-    if type(cb) ~= "function" then
-        error("callback must be a function", 2)
+    if cb ~= nil and type(cb) ~= "function" then
+        error("callback must be nil or a function", 2)
     end
 
     -- worker LRU cache retrieval
@@ -864,7 +874,13 @@ function _M:get(key, opts, cb, ...)
     end
 
     -- not in shm either
-    -- single worker must execute the callback
+
+    if cb == nil then
+        -- no L3 callback, early exit
+        return nil, nil, -1
+    end
+
+    -- L3 callback, single worker to run it
 
     return run_callback(self, key, namespaced_key, data, ttl, neg_ttl,
                         went_stale, l1_serializer, resurrect_ttl,
