@@ -17,21 +17,21 @@ package spanstore_test
 import (
 	"context"
 	"fmt"
-	"io/ioutil"
 	"log"
 	"math/rand"
 	"os"
+	"path/filepath"
 	"runtime/pprof"
 	"testing"
 	"time"
 
 	"github.com/stretchr/testify/assert"
 	"github.com/stretchr/testify/require"
-	"github.com/uber/jaeger-lib/metrics"
 	"go.uber.org/zap"
 
 	"github.com/jaegertracing/jaeger/model"
 	"github.com/jaegertracing/jaeger/pkg/config"
+	"github.com/jaegertracing/jaeger/pkg/metrics"
 	"github.com/jaegertracing/jaeger/plugin/storage/badger"
 	"github.com/jaegertracing/jaeger/storage/spanstore"
 )
@@ -74,7 +74,7 @@ func TestWriteReadBack(t *testing.T) {
 					},
 				}
 				err := sw.WriteSpan(context.Background(), &s)
-				assert.NoError(t, err)
+				require.NoError(t, err)
 			}
 		}
 
@@ -83,9 +83,9 @@ func TestWriteReadBack(t *testing.T) {
 				Low:  uint64(i),
 				High: 1,
 			})
-			assert.NoError(t, err)
+			require.NoError(t, err)
 
-			assert.Equal(t, spans, len(tr.Spans))
+			assert.Len(t, tr.Spans, spans)
 		}
 	})
 }
@@ -100,32 +100,32 @@ func TestValidation(t *testing.T) {
 
 		params.OperationName = "no-service"
 		_, err := sr.FindTraces(context.Background(), params)
-		assert.EqualError(t, err, "service name must be set")
+		require.EqualError(t, err, "service name must be set")
 		params.ServiceName = "find-service"
 
 		_, err = sr.FindTraces(context.Background(), nil)
-		assert.EqualError(t, err, "malformed request object")
+		require.EqualError(t, err, "malformed request object")
 
 		params.StartTimeMin = params.StartTimeMax.Add(1 * time.Hour)
 		_, err = sr.FindTraces(context.Background(), params)
-		assert.EqualError(t, err, "min start time is above max")
+		require.EqualError(t, err, "min start time is above max")
 		params.StartTimeMin = tid
 
 		params.DurationMax = time.Duration(1 * time.Millisecond)
 		params.DurationMin = time.Duration(1 * time.Minute)
 		_, err = sr.FindTraces(context.Background(), params)
-		assert.EqualError(t, err, "min duration is above max")
+		require.EqualError(t, err, "min duration is above max")
 
 		params = &spanstore.TraceQueryParameters{
 			StartTimeMin: tid,
 		}
 		_, err = sr.FindTraces(context.Background(), params)
-		assert.EqualError(t, err, "start and end time must be set")
+		require.EqualError(t, err, "start and end time must be set")
 
 		params.StartTimeMax = tid.Add(1 * time.Minute)
 		params.Tags = map[string]string{"A": "B"}
 		_, err = sr.FindTraces(context.Background(), params)
-		assert.EqualError(t, err, "service name must be set")
+		require.EqualError(t, err, "service name must be set")
 	})
 }
 
@@ -171,7 +171,7 @@ func TestIndexSeeks(t *testing.T) {
 				}
 
 				err := sw.WriteSpan(context.Background(), &s)
-				assert.NoError(t, err)
+				require.NoError(t, err)
 			}
 		}
 
@@ -189,24 +189,24 @@ func TestIndexSeeks(t *testing.T) {
 		}
 
 		trs, err := sr.FindTraces(context.Background(), params)
-		assert.NoError(t, err)
-		assert.Equal(t, 1, len(trs))
-		assert.Equal(t, spans, len(trs[0].Spans))
+		require.NoError(t, err)
+		assert.Len(t, trs, 1)
+		assert.Len(t, trs[0].Spans, spans)
 
 		params.OperationName = "operation-1"
 		trs, err = sr.FindTraces(context.Background(), params)
-		assert.NoError(t, err)
-		assert.Equal(t, 1, len(trs))
+		require.NoError(t, err)
+		assert.Len(t, trs, 1)
 
 		params.ServiceName = "service-10" // this should not match
 		trs, err = sr.FindTraces(context.Background(), params)
-		assert.NoError(t, err)
-		assert.Equal(t, 0, len(trs))
+		require.NoError(t, err)
+		assert.Empty(t, trs)
 
 		params.OperationName = "operation-4"
 		trs, err = sr.FindTraces(context.Background(), params)
-		assert.NoError(t, err)
-		assert.Equal(t, 0, len(trs))
+		require.NoError(t, err)
+		assert.Empty(t, trs)
 
 		// Multi-index hits
 
@@ -219,9 +219,9 @@ func TestIndexSeeks(t *testing.T) {
 		params.Tags = tags
 		params.DurationMin = time.Duration(1 * time.Millisecond)
 		trs, err = sr.FindTraces(context.Background(), params)
-		assert.NoError(t, err)
-		assert.Equal(t, 1, len(trs))
-		assert.Equal(t, spans, len(trs[0].Spans))
+		require.NoError(t, err)
+		assert.Len(t, trs, 1)
+		assert.Len(t, trs[0].Spans, spans)
 
 		// Query limited amount of hits
 
@@ -229,8 +229,8 @@ func TestIndexSeeks(t *testing.T) {
 		delete(params.Tags, "k11")
 		params.NumTraces = 2
 		trs, err = sr.FindTraces(context.Background(), params)
-		assert.NoError(t, err)
-		assert.Equal(t, 2, len(trs))
+		require.NoError(t, err)
+		assert.Len(t, trs, 2)
 		assert.Equal(t, traceOrder[59], trs[0].Spans[0].TraceID.Low)
 		assert.Equal(t, traceOrder[55], trs[1].Spans[0].TraceID.Low)
 		testOrder(trs)
@@ -244,8 +244,8 @@ func TestIndexSeeks(t *testing.T) {
 			NumTraces:    9,
 		}
 		trs, err = sr.FindTraces(context.Background(), params)
-		assert.NoError(t, err)
-		assert.Equal(t, 9, len(trs)) // Returns 23, we limited to 9
+		require.NoError(t, err)
+		assert.Len(t, trs, 9) // Returns 23, we limited to 9
 
 		// Check the newest items are returned
 		assert.Equal(t, traceOrder[50], trs[0].Spans[0].TraceID.Low)
@@ -257,8 +257,8 @@ func TestIndexSeeks(t *testing.T) {
 		params.DurationMax = 0
 		params.NumTraces = 7
 		trs, err = sr.FindTraces(context.Background(), params)
-		assert.NoError(t, err)
-		assert.Equal(t, 7, len(trs))
+		require.NoError(t, err)
+		assert.Len(t, trs, 7)
 		assert.Equal(t, traceOrder[59], trs[0].Spans[0].TraceID.Low)
 		assert.Equal(t, traceOrder[53], trs[6].Spans[0].TraceID.Low)
 		testOrder(trs)
@@ -270,9 +270,9 @@ func TestIndexSeeks(t *testing.T) {
 		}
 
 		trs, err = sr.FindTraces(context.Background(), params)
-		assert.NoError(t, err)
-		assert.Equal(t, 5, len(trs))
-		assert.Equal(t, spans, len(trs[0].Spans))
+		require.NoError(t, err)
+		assert.Len(t, trs, 5)
+		assert.Len(t, trs[0].Spans, spans)
 		testOrder(trs)
 
 		// StartTime and Duration queries
@@ -281,8 +281,8 @@ func TestIndexSeeks(t *testing.T) {
 		params.DurationMax = time.Duration(56 * time.Millisecond) // trace 56 (max)
 
 		trs, err = sr.FindTraces(context.Background(), params)
-		assert.NoError(t, err)
-		assert.Equal(t, 6, len(trs))
+		require.NoError(t, err)
+		assert.Len(t, trs, 6)
 		assert.Equal(t, traceOrder[56], trs[0].Spans[0].TraceID.Low)
 		assert.Equal(t, traceOrder[51], trs[5].Spans[0].TraceID.Low)
 		testOrder(trs)
@@ -299,8 +299,8 @@ func TestFindNothing(t *testing.T) {
 		}
 
 		trs, err := sr.FindTraces(context.Background(), params)
-		assert.NoError(t, err)
-		assert.Len(t, trs, 0)
+		require.NoError(t, err)
+		assert.Empty(t, trs)
 
 		tr, err := sr.GetTrace(context.Background(), model.TraceID{High: 0, Low: 0})
 		assert.Equal(t, spanstore.ErrTraceNotFound, err)
@@ -329,7 +329,7 @@ func TestWriteDuplicates(t *testing.T) {
 					Duration:  time.Duration(i + j),
 				}
 				err := sw.WriteSpan(context.Background(), &s)
-				assert.NoError(t, err)
+				require.NoError(t, err)
 			}
 		}
 	})
@@ -357,7 +357,7 @@ func TestMenuSeeks(t *testing.T) {
 					Duration:  time.Duration(i + j),
 				}
 				err := sw.WriteSpan(context.Background(), &s)
-				assert.NoError(t, err)
+				require.NoError(t, err)
 			}
 		}
 
@@ -365,20 +365,18 @@ func TestMenuSeeks(t *testing.T) {
 			context.Background(),
 			spanstore.OperationQueryParameters{ServiceName: "service-1"},
 		)
-		assert.NoError(t, err)
+		require.NoError(t, err)
 
 		serviceList, err := sr.GetServices(context.Background())
-		assert.NoError(t, err)
+		require.NoError(t, err)
 
-		assert.Equal(t, spans, len(operations))
-		assert.Equal(t, services, len(serviceList))
+		assert.Len(t, operations, spans)
+		assert.Len(t, serviceList, services)
 	})
 }
 
 func TestPersist(t *testing.T) {
-	dir, err := ioutil.TempDir("", "badgerTest")
-	assert.NoError(t, err)
-	defer os.RemoveAll(dir)
+	dir := t.TempDir()
 
 	p := func(t *testing.T, dir string, test func(t *testing.T, sw spanstore.Writer, sr spanstore.Reader)) {
 		f := badger.NewFactory()
@@ -398,16 +396,16 @@ func TestPersist(t *testing.T) {
 			keyParam,
 			valueParam,
 		})
-		f.InitFromViper(v)
+		f.InitFromViper(v, zap.NewNop())
 
-		err = f.Initialize(metrics.NullFactory, zap.NewNop())
-		assert.NoError(t, err)
+		err := f.Initialize(metrics.NullFactory, zap.NewNop())
+		require.NoError(t, err)
 
 		sw, err := f.CreateSpanWriter()
-		assert.NoError(t, err)
+		require.NoError(t, err)
 
 		sr, err := f.CreateSpanReader()
-		assert.NoError(t, err)
+		require.NoError(t, err)
 
 		test(t, sw, sr)
 	}
@@ -427,7 +425,7 @@ func TestPersist(t *testing.T) {
 			Duration:  time.Duration(1 * time.Hour),
 		}
 		err := sw.WriteSpan(context.Background(), &s)
-		assert.NoError(t, err)
+		require.NoError(t, err)
 	})
 
 	p(t, dir, func(t *testing.T, sw spanstore.Writer, sr spanstore.Reader) {
@@ -435,12 +433,12 @@ func TestPersist(t *testing.T) {
 			Low:  uint64(1),
 			High: 1,
 		})
-		assert.NoError(t, err)
+		require.NoError(t, err)
 		assert.Equal(t, "operation-p", trace.Spans[0].OperationName)
 
 		services, err := sr.GetServices(context.Background())
-		assert.NoError(t, err)
-		assert.Equal(t, 1, len(services))
+		require.NoError(t, err)
+		assert.Len(t, services, 1)
 	})
 }
 
@@ -457,16 +455,16 @@ func runFactoryTest(tb testing.TB, test func(tb testing.TB, sw spanstore.Writer,
 		"--badger.ephemeral=true",
 		"--badger.consistency=false",
 	})
-	f.InitFromViper(v)
+	f.InitFromViper(v, zap.NewNop())
 
 	err := f.Initialize(metrics.NullFactory, zap.NewNop())
-	assert.NoError(tb, err)
+	require.NoError(tb, err)
 
 	sw, err := f.CreateSpanWriter()
-	assert.NoError(tb, err)
+	require.NoError(tb, err)
 
 	sr, err := f.CreateSpanReader()
-	assert.NoError(tb, err)
+	require.NoError(tb, err)
 
 	test(tb, sw, sr)
 }
@@ -575,7 +573,6 @@ func makeReadBenchmark(b *testing.B, tid time.Time, params *spanstore.TraceQuery
 		}
 		b.StopTimer()
 	})
-
 }
 
 func BenchmarkServiceTagsRangeQueryLimitIndexFetch(b *testing.B) {
@@ -610,14 +607,14 @@ func BenchmarkServiceIndexLimitFetch(b *testing.B) {
 
 // Opens a badger db and runs a test on it.
 func runLargeFactoryTest(tb testing.TB, test func(tb testing.TB, sw spanstore.Writer, sr spanstore.Reader)) {
-	assert := assert.New(tb)
+	assertion := require.New(tb)
 	f := badger.NewFactory()
 	opts := badger.NewOptions("badger")
 	v, command := config.Viperize(opts.AddFlags)
 
-	dir := "/mnt/ssd/badger/testRun"
-	err := os.MkdirAll(dir, 0700)
-	assert.NoError(err)
+	dir := filepath.Join(tb.TempDir(), "badger-testRun")
+	err := os.MkdirAll(dir, 0o700)
+	assertion.NoError(err)
 	keyParam := fmt.Sprintf("--badger.directory-key=%s", dir)
 	valueParam := fmt.Sprintf("--badger.directory-value=%s", dir)
 
@@ -628,16 +625,16 @@ func runLargeFactoryTest(tb testing.TB, test func(tb testing.TB, sw spanstore.Wr
 		valueParam,
 	})
 
-	f.InitFromViper(v)
+	f.InitFromViper(v, zap.NewNop())
 
 	err = f.Initialize(metrics.NullFactory, zap.NewNop())
-	assert.NoError(err)
+	assertion.NoError(err)
 
 	sw, err := f.CreateSpanWriter()
-	assert.NoError(err)
+	assertion.NoError(err)
 
 	sr, err := f.CreateSpanReader()
-	assert.NoError(err)
+	assertion.NoError(err)
 
 	defer func() {
 		err := f.Close()
@@ -671,7 +668,7 @@ func TestRandomTraceID(t *testing.T) {
 			Duration:  1 * time.Second,
 		}
 		err := sw.WriteSpan(context.Background(), &s1)
-		assert.NoError(t, err)
+		require.NoError(t, err)
 
 		s2 := model.Span{
 			TraceID: model.TraceID{
@@ -694,7 +691,7 @@ func TestRandomTraceID(t *testing.T) {
 			Duration:  1 * time.Second,
 		}
 		err = sw.WriteSpan(context.Background(), &s2)
-		assert.NoError(t, err)
+		require.NoError(t, err)
 
 		params := &spanstore.TraceQueryParameters{
 			StartTimeMin: time.Now().Add(-1 * time.Minute),
@@ -705,7 +702,7 @@ func TestRandomTraceID(t *testing.T) {
 			},
 		}
 		traces, err := sr.FindTraces(context.Background(), params)
-		assert.NoError(t, err)
-		assert.Equal(t, 1, len(traces))
+		require.NoError(t, err)
+		assert.Len(t, traces, 1)
 	})
 }
