@@ -60,9 +60,11 @@
 #if defined(CONFIG_DEBUG_FS)
 #include <linux/fs.h>
 #endif
+#include "qed_dbg_hsi.h"
 #include "qed_debug.h"
 #include "qed_hsi.h"
 #include "qed_if.h"
+#include "qed_mfw_hsi.h"
 #include "qed_compat.h"
 #include "qed_eth_if.h"
 #include "qed_if.h"
@@ -70,8 +72,8 @@
 #define _QED_H_
 
 #define QED_MAJOR_VERSION               8
-#define QED_MINOR_VERSION               53
-#define QED_REVISION_VERSION            0
+#define QED_MINOR_VERSION               70
+#define QED_REVISION_VERSION            12
 #define QED_ENGINEERING_VERSION 0
 
 #define QED_VERSION						 \
@@ -82,13 +84,21 @@
 	((FW_MAJOR_VERSION << 24) | (FW_MINOR_VERSION << 16) | \
 	 (FW_REVISION_VERSION << 8) | FW_ENGINEERING_VERSION)
 
-#define IS_QED_PACING(p_hwfn) \
-	(!!(p_hwfn->b_en_pacing))
+#define IS_QED_PACING(p_hwfn)   (!!(p_hwfn->b_en_pacing))
+#define IS_QED_DCQCN(p_hwfn)    (!!(p_hwfn->b_en_dcqcn))
 
 #define MAX_HWFNS_PER_DEVICE    2
 #define NAME_SIZE 16
 #define ARRAY_DECL static const
-#define QED_WFQ_UNIT    100
+
+#define QM_INVALID_PQ_ID                0xffff
+#define QM_INVALID_WFQ_ID               0xffff
+#define QM_INVALID_RL_ID                0xffff
+
+#define QED_WFQ_RESOLUTION      10000
+#define QED_WFQ_UNIT    (QED_WFQ_RESOLUTION + COMMON_MAX_QM_GLOBAL_RLS)
+#define QED_WEIGHT_IN_WFQ_GRANULARITY(__percentage) \
+	(QED_WFQ_UNIT * __percentage / 100)
 
 /* Constants */
 #define QED_WID_SIZE            (1024)
@@ -438,12 +448,29 @@ struct qed_offload_pq {
 	u8 tc;
 };
 
+enum qed_pqset_aux_offld_type {
+	QED_AUX_OFFLOAD_SHARED_PQ_SET,
+	QED_AUX_OFFLOAD_DEDICATED_PQ_SET,
+	QED_AUX_OFFLOAD_SHARED_DEFAULT,
+
+	QED_AUX_OFFLOAD_PQ_SET_MAX
+};
+
+enum qed_pqset_offld_type {
+	QED_OFFLOAD_SHARED_PQ_SET,
+	QED_OFFLOAD_DEDICATED_PQ_SET,
+
+	QED_OFFLOAD_PQ_SET_MAX
+};
+
 struct qed_qm_info {
 	struct init_qm_pq_params *qm_pq_params;
 	struct init_qm_vport_params *qm_vport_params;
+	struct init_qm_rl_params *qm_rl_params;
 	struct init_qm_port_params *qm_port_params;
 	u16 start_pq;
-	u8 start_vport;
+	u16 start_vport;
+	u16 start_rl;
 	u16 pure_lb_pq;
 	u16 first_ofld_pq;
 	u16 first_llt_pq;
@@ -457,7 +484,8 @@ struct qed_qm_info {
 	u16 num_pqs;
 	u16 num_vf_pqs;
 	u16 ilt_pf_pqs;
-	u8 num_vports;
+	u16 num_vports;
+	u16 num_rls;
 	u8 max_phys_tcs_per_port;
 	u8 ooo_tc;
 	bool pq_overflow;
@@ -467,13 +495,26 @@ struct qed_qm_info {
 	bool vport_wfq_en;
 	bool vf_rdma_en;
 #define IS_QED_QM_VF_RDMA(_p_hwfn) ((_p_hwfn)->qm_info.vf_rdma_en)
-	u8 pf_wfq;
+	u16 pf_wfq;
 	u32 pf_rl;
 	struct qed_wfq_data *wfq_data;
 	u8 num_pf_rls;
 	struct qed_offload_pq offload_group[OFLD_GRP_SIZE];
 	u8 offload_group_count;
 #define IS_QED_OFLD_GRP(p_hwfn) (p_hwfn->qm_info.offload_group_count > 0)
+	u16 num_pqset;
+#define QED_PQSET_SUPPORTED(p_hwfn)     (!!(p_hwfn)->qm_info.num_pqset)
+#define QED_PQSET_NUM_OF_PQ_TYPES       3
+	u16 start_pqset_num;
+#define QED_IS_SHARED_DEFAULT_PQSET(_p_hwfn, _pqs_id) \
+	(((_pqs_id) == 0) && ((_p_hwfn)->qm_info.start_pqset_num != 0))
+	u16 first_pqset_pq;
+	u8 pqset_num_offload_pqs;
+	u8 pqset_num_aux_pqs;
+	enum qed_pqset_offld_type offload_type;
+	enum qed_pqset_aux_offld_type aux_offload_type;
+	u16 num_pqs_per_pqset;
+	u8 relative_wfq_speed;
 
 	/* Locks PQ getters against QM info initialization */
 	spinlock_t qm_info_lock;
@@ -496,6 +537,9 @@ struct qed_db_recovery_info {
 
 	/* Indicates that DORQ attention was handled in qed_int_deassertion */
 	bool dorq_attn;
+
+	/* Indicates that doorbell recovery mechanism setup is done */
+	bool setup_done;
 };
 
 struct storm_stats {
@@ -559,6 +603,12 @@ enum qed_mf_mode_bit {
 
 	/* Allow RoCE LAG */
 	QED_MF_ROCE_LAG,
+
+	/* QinQ support */
+	QED_MF_QINQ_SPECIFIC,
+
+	/* Large ILT size */
+	QED_MF_LARGE_ILT,
 };
 
 enum qed_ufp_mode {
@@ -639,10 +689,10 @@ enum qed_hsi_def_type {
 	__stringify(QED_ENGINEERING_VERSION)
 
 #ifndef QED_UPSTREAM
-#define QED_ETH_INTERFACE_VERSION       5300
-#define QED_ISCSI_INTERFACE_VERSION     5300
-#define QED_FCOE_INTERFACE_VERSION      5300
-#define QED_ROCE_INTERFACE_VERSION      5300
+#define QED_ETH_INTERFACE_VERSION       7012
+#define QED_ISCSI_INTERFACE_VERSION     7012
+#define QED_FCOE_INTERFACE_VERSION      7012
+#define QED_ROCE_INTERFACE_VERSION      7012
 #endif
 
 struct qed_simd_fp_handler {
@@ -675,6 +725,8 @@ struct qed_hwfn {
 
 	u32 dp_module;
 	u8 dp_level;
+	u32 dp_int_module;
+	u8 dp_int_level;
 	char name[NAME_SIZE];
 	void *dp_ctx;
 
@@ -784,6 +836,12 @@ struct qed_hwfn {
 	 * struct qed_hw_prepare_params by qed client.
 	 */
 	bool b_en_pacing;
+
+	/* Enable DCQCN and skip IOV configuration. Reflects the value
+	 * requested in hw_prepare_params by the QED client.
+	 */
+	bool b_en_dcqcn;
+
 	struct qed_lag_info lag_info;
 
 	/* Nvm images number and attributes */
@@ -792,6 +850,16 @@ struct qed_hwfn {
 	struct phys_mem_desc *fw_overlay_mem;
 	int (*p_dummy_cb)
 	 (struct qed_hwfn *, void *);
+
+	/* TC value to be used in QinQ header */
+	u8 qinq_tc;
+	spinlock_t *qm_lock;
+
+	/* retry count to acquire MCP resource lock */
+	u8 mcp_resc_lock_retry_cnt;
+
+	/* Skip MCP Drain if it is already in progress */
+	bool mcp_drain_inprogress;
 	struct qed_ptt *p_arfs_ptt;
 	struct qed_simd_fp_handler simd_proto_handler[64];
 
@@ -893,7 +961,12 @@ struct qed_dev {
 	u8 dp_level;
 	char name[NAME_SIZE];
 	void *dp_ctx;
+	struct qed_internal_trace internal_trace;
+	u8 dp_int_level;
+	u32 dp_int_module;
 
+/* for work DP_* macros with cdev, hwfn, etc */
+	struct qed_dev *cdev;
 	enum qed_dev_type type;
 /* Translate type/revision combo into the proper conditions */
 #define QED_IS_BB(dev)  ((dev)->type == QED_DEV_TYPE_BB)
@@ -1058,7 +1131,17 @@ struct qed_dev {
 	/* Indicates whether this PF serves a storage target */
 	bool b_is_target;
 
+	/* AER in progress */
+	bool aer_in_prog;
+
+	/* PCI Host reset assertions received by mfw */
+	u16 hot_reset_count;
+
+	/* Instruct driver to read statistics from the specified bin id */
+	u8 stats_bin_id;
+
 	/* Linux specific here */
+	struct qed_dev_info common_dev_info;
 	struct pci_dev *pdev;
 	u32 flags;
 #define QED_FLAG_STORAGE_STARTED (1 << 0)	/* TODO - very likely this isn't the
@@ -1095,13 +1178,16 @@ struct qed_dev {
 	struct qed_dbg_feature dbg_features[DBG_FEATURE_NUM];
 	struct dentry *bdf_dentry;
 	bool test_result_available;
+	struct kobject *bdf_kobj;
 #define QED_TEST_RESULT_LENGTH  11	/* 10 chars for max int in decimal + '\0' */
 	char test_result[QED_TEST_RESULT_LENGTH + 1];	/* +1 for '\n' */
 	struct chain_print_struct chain_info;
 	u8 engine_for_debug;
 	bool disable_ilt_dump;
 	bool recording_active;
+	bool disable_internal_trace_dump;
 	struct qed_dbg_mdump dbg_mdump;
+	u8 dbg_bin_dump;
 #endif
 
 	 DECLARE_HASHTABLE(connections, 10);
@@ -1118,7 +1204,7 @@ struct qed_dev {
 #ifndef QED_UPSTREAM
 	bool b_dump_dbg_data;
 	u8 dbg_data_path[PATH_MAX];
-#define QED_DBG_DATA_FILE_NAME_SIZE     40
+#define QED_DBG_DATA_FILE_NAME_SIZE     60
 #define QED_DBG_DATA_PATH_MAX_SIZE      (PATH_MAX - QED_DBG_DATA_FILE_NAME_SIZE)
 #endif
 
@@ -1132,10 +1218,15 @@ struct qed_dev {
 #endif
 	bool tx_switching;
 	enum qed_vf_mac_origin vf_mac_origin;
-	bool dcqcn_enable;
+
+#ifdef _HAS_DEVLINK		/* QED_UPSTREAM */
+	bool iwarp_cmt;
+#endif
+	/* Indiactes whether cdev needs to be retained during recovery or not */
+	bool b_reuse_dev;
 };
 
-u32 qed_get_hsi_def_val(struct qed_dev *cdev, enum qed_hsi_def_type type);
+u16 qed_get_hsi_def_val(struct qed_dev *cdev, enum qed_hsi_def_type type);
 
 #define NUM_OF_VFS(dev)	\
 	qed_get_hsi_def_val(dev, QED_HSI_DEF_MAX_NUM_VFS)
@@ -1164,6 +1255,9 @@ u32 qed_get_hsi_def_val(struct qed_dev *cdev, enum qed_hsi_def_type type);
 #define NUM_OF_BTB_BLOCKS(dev) \
 	qed_get_hsi_def_val(dev, QED_HSI_DEF_MAX_BTB_BLOCKS)
 
+#define QED_RECOV_IN_PROG(_cdev) \
+	((_cdev)->recov_in_prog || (_cdev)->aer_in_prog)
+
 /**
  * @brief qed_concrete_to_sw_fid - get the sw function id from
  *        the concrete value.
@@ -1223,20 +1317,44 @@ void qed_set_fw_mac_addr(__le16 * fw_msb,
 #define PQ_FLAGS_MTC    (1 << 9)
 #define PQ_FLAGS_VFR    BIT(10)
 #define PQ_FLAGS_VSR    BIT(11)
+#define PQ_FLAGS_PQSET  BIT(12)
 
+#define QED_DEFAULT_PQ_SET      0
+u16 qed_get_pq_idx(struct qed_hwfn *p_hwfn, u32 pq_flags, u16 pq_set, u8 tc);
 /* physical queue index for cm context intialization */
 u16 qed_get_cm_pq_idx(struct qed_hwfn *p_hwfn, u32 pq_flags);
-u16 qed_get_cm_pq_idx_mcos(struct qed_hwfn *p_hwfn, u8 tc);
-u16 qed_get_cm_pq_idx_vf(struct qed_hwfn *p_hwfn, u16 vf);
-u16 qed_get_cm_pq_idx_vf_rdma(struct qed_hwfn *p_hwfn, u16 vf);
+
+u16 qed_get_cm_pq_idx_tc(struct qed_hwfn *p_hwfn,
+			 u32 pq_flags, u16 pq_set, u8 tc);
+
+u16 qed_get_cm_pq_idx_vf(struct qed_hwfn *p_hwfn, u16 vf, u16 pq_set_id, u8 tc);
+
+u16 qed_get_cm_pq_idx_vf_ll2(struct qed_hwfn *p_hwfn,
+			     u16 vf, u16 pq_set_id, u8 tc);
+
+u16 qed_get_cm_pq_idx_vf_rdma(struct qed_hwfn *p_hwfn,
+			      u16 vf, u8 tc, u16 pq_set_id);
+
+u16 qed_get_cm_pq_idx_vf_rdma_llt(struct qed_hwfn *p_hwfn,
+				  u16 vf, u8 tc, u16 pq_set_id);
 
 u16 qed_get_cm_pq_idx_rl(struct qed_hwfn *p_hwfn, u16 rl);
+
 u16 qed_get_cm_pq_idx_grp(struct qed_hwfn *p_hwfn, u8 idx);
-u16 qed_get_cm_pq_idx_ofld_mtc(struct qed_hwfn *p_hwfn, u16 idx, u8 tc);
-u16 qed_get_cm_pq_idx_llt_mtc(struct qed_hwfn *p_hwfn, u16 idx, u8 tc);
+
+u16 qed_get_cm_pq_idx_ofld_mtc(struct qed_hwfn *p_hwfn,
+			       u16 idx, u8 tc, u16 pq_set_id);
+
+u16 qed_get_cm_pq_idx_llt_mtc(struct qed_hwfn *p_hwfn,
+			      u16 idx, u8 tc, u16 pq_set_id);
+
 u16 qed_get_cm_pq_idx_ll2(struct qed_hwfn *p_hwfn, u8 tc);
-/* qm vport for rate limit configuration */
-u16 qed_get_qm_vport_idx_rl(struct qed_hwfn *p_hwfn, u16 rl);
+
+/* qm vport/rl for rate limit configuration */
+u16 qed_get_pq_vport_id_from_rl(struct qed_hwfn *p_hwfn, u16 rl);
+u16 qed_get_pq_vport_id_from_vf(struct qed_hwfn *p_hwfn, u16 vf);
+u16 qed_get_pq_rl_id_from_rl(struct qed_hwfn *p_hwfn, u16 rl);
+u16 qed_get_pq_rl_id_from_vf(struct qed_hwfn *p_hwfn, u16 vf);
 
 const char *qed_hw_get_resc_name(enum qed_resources res_id);
 
@@ -1265,6 +1383,9 @@ u16 qed_init_qm_get_num_pqs(struct qed_hwfn *p_hwfn);
 
 void qed_hw_info_set_offload_tc(struct qed_hw_info *p_info, u8 tc);
 u8 qed_get_offload_tc(struct qed_hwfn *p_hwfn);
+int
+qed_pq_set_get_pqs_of_tc(struct qed_hwfn *p_hwfn,
+			 u16 pq_set, u8 tc, u16 * pqs, u8 num_pqs);
 
 #define MFW_PORT(_p_hwfn)       ((_p_hwfn)->abs_pf_id %	\
 				 qed_device_num_ports((_p_hwfn)->cdev))
@@ -1312,6 +1433,12 @@ void qed_set_platform_str(struct qed_hwfn *p_hwfn, char *buf_str, u32 buf_size);
 #define USTORM_QZONE_START(dev) (MSTORM_QZONE_START(dev) + \
 				 (MSTORM_QZONE_SIZE * NUM_OF_L2_QUEUES(dev)))
 
+#define GET_GTT_REG_ADDR(__base, __offset, __idx) \
+	((__base) + __offset ## _GTT_OFFSET((__idx)))
+
+#define GET_GTT_BDQ_REG_ADDR(__base, __offset, __idx, __bdq_idx) \
+	((__base) + __offset ## _GTT_OFFSET((__idx), (__bdq_idx)))
+
 /* Other Linux specific common definitions */
 #define DP_NAME(cdev) ((cdev)->name)
 #define DP_TRACK(cdev) (0)
@@ -1320,17 +1447,16 @@ void qed_set_platform_str(struct qed_hwfn *p_hwfn, char *buf_str, u32 buf_size);
 							 (cdev->regview) + \
 							 (offset))
 
-#define REG_RD(cdev, offset)            readl(REG_ADDR(cdev, offset))
-#define REG_RD8(cdev, offset)           readb(REG_ADDR(cdev, offset))
-#define REG_RD16(cdev, offset)          readw(REG_ADDR(cdev, offset))
+#define REG_RD(cdev, offset)            ({ rmb(); readl(REG_ADDR(cdev, offset)); })
+#define REG_RD8(cdev, offset)           ({ rmb(); readb(REG_ADDR(cdev, offset)); })
+#define REG_RD16(cdev, offset)          ({ rmb(); readw(REG_ADDR(cdev, offset)); })
 
-#define REG_WR(cdev, offset, val)       writel((u32)val, REG_ADDR(cdev, offset))
-#define REG_WR8(cdev, offset, val)      writeb((u8)val, REG_ADDR(cdev, offset))
-#define REG_WR16(cdev, offset, val)     writew((u16)val, REG_ADDR(cdev, offset))
-
-#define DOORBELL(cdev, db_addr, val)			 \
-	writel((u32)val, (void __iomem *)((u8 __iomem *) \
-					  (cdev->doorbells) + (db_addr)))
+#define REG_WR(cdev, offset, \
+	       val)       ({ writel((u32)val, REG_ADDR(cdev, offset)); wmb(); })
+#define REG_WR8(cdev, offset, \
+		val)      ({ writeb((u8)val, REG_ADDR(cdev, offset)); wmb(); })
+#define REG_WR16(cdev, offset, \
+		 val)     ({ writew((u16)val, REG_ADDR(cdev, offset)); wmb(); })
 
 #define QED_DB_SHIFT    5	/* 32 bytes - check if correct for E4 */
 
@@ -1344,6 +1470,7 @@ void qed_transceiver_rx_los(struct qed_hwfn *p_hwfn);
 void qed_dcbx_aen(struct qed_hwfn *hwfn, u32 mib_type);
 u32 qed_unzip_data(struct qed_hwfn *p_hwfn,
 		   u32 input_len, u8 * input_buf, u32 max_size, u8 * unzip_buf);
+int qed_recovery_process(struct qed_dev *cdev);
 void qed_schedule_recovery_handler(struct qed_hwfn *p_hwfn);
 void qed_hw_error_occurred(struct qed_hwfn *p_hwfn,
 			   enum qed_hw_err_type err_type);
@@ -1375,5 +1502,9 @@ u8 qed_crc8(const u8 table[CRC8_TABLE_SIZE], u8 * pdata, size_t nbytes, u8 crc);
 
 unsigned long qed_get_epoch_time(void);
 
+extern const struct qed_common_ops qed_common_ops_pass;
+#define QED_INT_DBG_STORE(P_DEV, ...) \
+	qed_common_ops_pass.internal_trace(P_DEV->cdev, __VA_ARGS__)
+
 #endif /* _QED_H_ */
 #endif
