@@ -35,6 +35,12 @@
 #include <linux/version.h>
 #include <linux/skbuff.h>
 #include <linux/list.h>
+#ifdef _HAS_DEVLINK
+#include <net/devlink.h>
+#endif
+
+#define HAS_NDO(feat) \
+	(defined(_HAS_NDO_ ## feat) || defined (_HAS_NDO_EXT_ ## feat))
 
 #ifndef RHEL_RELEASE_VERSION
 #define RHEL_RELEASE_VERSION(a, b) 0
@@ -59,9 +65,25 @@
 #define QED_ENC_SUPPORTED 1
 #endif
 
+#ifndef SLES_DISTRO
+#ifdef CONFIG_SUSE_KERNEL
+#if (LINUX_VERSION_CODE >= KERNEL_VERSION(5, 3, 18))
+/* SLES15 SP2 is at least 5.3.18+ based */
+#define SLES_DISTRO 0x1502
+#elif (LINUX_VERSION_CODE >= KERNEL_VERSION(4, 12, 14))
+/* SLES15 is at least 4.12.14+ based */
+#define SLES_DISTRO 0x1502
+#endif
+#endif /* CONFIG_SUSE_KERNEL */
+#endif
+
 #define NOT_SLES_OR_PRE_VERSION(a) \
 	(!defined(SLES_DISTRO) || (SLES_DISTRO < (a)))
+#define SLES_STARTING_AT_VERSION(a) \
+	(defined(SLES_DISTRO) && (SLES_DISTRO >= (a)))
 #define SLES11_SP3	0x1103
+#define SLES15_SP2	0x1502
+#define SLES15_SP3	0x1503
 
 #if (LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 20))
 #define atomic_read(&pdev->enable_cnt) 1
@@ -76,6 +98,9 @@
 	ioremap_nocache(cdev->pci_params.mem_start, pci_resource_len(pdev, 0))
 #endif
 
+#if (LINUX_VERSION_CODE > KERNEL_VERSION(5, 5, 0))
+#define ioremap_nocache ioremap
+#endif
 
 #if (LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 34))
 #define dma_set_mask(dev, x) 		\
@@ -698,6 +723,7 @@ static inline void writeq(u64 value, volatile void __iomem *addr)
 {
 	writel((u32)value, addr);
 	writel((u32)(value >> 32), addr + 4);
+	wmb();
 }
 #endif
 
@@ -705,6 +731,8 @@ static inline void writeq(u64 value, volatile void __iomem *addr)
 #define readq readq
 static inline u64 readq(void __iomem *addr)
 {
+	rmb();
+
 	return readl(addr) | (((u64) readl(addr + 4)) << 32LL);
 }
 #endif
@@ -728,6 +756,10 @@ static inline void eth_zero_addr(u8 *addr)
 }
 #endif
 
+#ifndef _HAS_TIME_T
+#define time_t time64_t
+#endif
+
 #ifndef _HAS_TIME_TO_TM
 static inline void time_to_tm(time_t totalsecs, int offset, struct tm *result)
 {
@@ -735,6 +767,17 @@ static inline void time_to_tm(time_t totalsecs, int offset, struct tm *result)
 }
 #endif
 
+#ifndef _HAS_TIMESPEC64
+#define timespec64 timespec
+#define ktime_get_real_ts64 getnstimeofday
+#endif
+
+#if defined(_HAS_SECURE_BOOT)  && defined(_HAS_EFI_ENABLED)
+#define _efi_enabled efi_enabled(EFI_SECURE_BOOT)
+#else
+#define _efi_enabled 0
+#endif
+
 #ifdef DEFINE_DATA_ACCESS_EXCEEDS_WORD_SIZE
 static __always_inline void data_access_exceeds_word_size(void)
 #ifdef __compiletime_warning
@@ -787,4 +830,54 @@ static __always_inline void __read_once_size(volatile void *p, void *res, int si
 #define ETH_P_8021AD	0x88A8
 #endif
 
+#ifdef __has_attribute
+#ifndef __GCC4_has_attribute___fallthrough__
+#define __GCC4_has_attribute___fallthrough__ 0
+#endif
+#if __has_attribute(__fallthrough__)
+#define COMPAT_FALLTHROUGH   __attribute__((__fallthrough__))
+#else
+#define COMPAT_FALLTHROUGH
+#endif
+#else
+#define COMPAT_FALLTHROUGH
+#endif
+
+#if (LINUX_PRE_VERSION(5, 9, 0) && NOT_RHEL_OR_PRE_VERSION(8, 4)) && NOT_SLES_OR_PRE_VERSION(SLES15_SP3)
+#define devlink_info_board_serial_number_put  devlink_info_serial_number_put
+#endif
+#if (LINUX_PRE_VERSION(5, 7, 0) && NOT_RHEL_OR_PRE_VERSION(8, 3)) && NOT_SLES_OR_PRE_VERSION(SLES15_SP3)
+#ifdef _HAS_DEVLINK_AUTO_RECOVER
+#define devlink_health_reporter_create(A, B, C, D)  (devlink_health_reporter_create)((A), (B), (C), true, (D))
+#endif
+#endif
+#ifndef DEVLINK_INFO_VERSION_GENERIC_FW_ROCE
+#define DEVLINK_INFO_VERSION_GENERIC_FW_ROCE	"fw.roce"
+#endif
+
+static inline int __qed_devlink_register(void *dl,
+					 struct device *dev)
+{
+#ifdef _HAS_DEVLINK
+	struct devlink *devlink = (struct devlink *)dl;
+	int rc = 0;
+
+#ifdef _HAS_DEVLINK_STRUCT_DEV_DEVLINK_REGISTER
+#ifdef _HAS_DEVLINK_INT_DEVLINK_REGISTER
+	rc =  devlink_register(devlink, dev);
+#else
+	devlink_register(devlink, dev);
+#endif
+#else
+#ifdef _HAS_DEVLINK_INT_DEVLINK_REGISTER
+	rc = devlink_register(devlink);
+#else
+	devlink_register(devlink);
+#endif
+#endif
+	return rc;
+#else
+	return 0;
+#endif
+}
 #endif /* _QED_COMPAT_H_ */
