@@ -252,7 +252,8 @@ struct pfvf_acquire_resp_tlv {
 		u8 os_type;
 
 		/* These should match the PF's qed_dev values */
-		u16 chip_rev;
+		u8 chip_rev;
+		u8 chip_metal;
 		u8 dev_type;
 
 		/* Doorbell bar size configured in HW: log(size) or 0 */
@@ -317,6 +318,24 @@ struct vfpf_soft_flr_tlv {
 	u32 reserved2;
 };
 
+/* VFQ filter request */
+struct vfpf_filter_cfg_tlv {
+	/* from qed_ntuple_filter_params */
+	struct vfpf_first_tlv first_tlv;
+	u32 length;
+	u32 qid;
+
+	/* Enough packet_hdr size to ensure
+	 * template pkt hdr for the rule can fit in.
+	 */
+#define MAX_PKT_HDR_LEN 256
+	u8 packet_hdr_buf[MAX_PKT_HDR_LEN];
+	u8 b_is_add;
+	u8 b_is_drop;
+	u8 mode;
+	u8 padding[5];
+};
+
 /* Setup Queue */
 struct vfpf_start_rxq_tlv {
 	struct vfpf_first_tlv first_tlv;
@@ -350,7 +369,8 @@ struct vfpf_start_txq_tlv {
 	u32 flags;		/* VFPF_QUEUE_FLG_X flags */
 	u16 hc_rate;		/* desired interrupts per sec. */
 	u8 sb_index;
-	u8 padding[3];
+	u8 tc;
+	u8 padding[2];
 };
 
 /* Stop RX Queue */
@@ -1262,6 +1282,7 @@ union vfpf_tlvs {
 	struct vfpf_rdma_query_port_tlv rdma_query_device;
 	struct vfpf_async_event_req_tlv async_event_req;
 	struct vfpf_soft_flr_tlv vf_soft_flr;
+	struct vfpf_filter_cfg_tlv filter_cfg;
 	struct tlv_buffer_size tlv_buf_size;
 };
 
@@ -1450,6 +1471,7 @@ enum {
 	CHANNEL_TLV_RDMA_MODIFY_SRQ,
 	CHANNEL_TLV_RDMA_DESTROY_SRQ,
 	CHANNEL_TLV_SOFT_FLR,
+	CHANNEL_TLV_FILTER_CFG,
 	CHANNEL_TLV_MAX,
 
 	/* Required for iterating over vport-update tlvs.
@@ -2011,6 +2033,7 @@ int qed_vf_pf_rxq_start(struct qed_hwfn *p_hwfn,
 int
 qed_vf_pf_txq_start(struct qed_hwfn *p_hwfn,
 		    struct qed_queue_cid *p_cid,
+		    u8 tc,
 		    dma_addr_t pbl_addr,
 		    u16 pbl_size, void __iomem ** pp_doorbell);
 
@@ -2125,6 +2148,10 @@ int qed_vf_pf_filter_ucast(struct qed_hwfn *p_hwfn,
 void qed_vf_pf_filter_mcast(struct qed_hwfn *p_hwfn,
 			    struct qed_filter_mcast *p_filter_cmd);
 
+int
+qed_vf_pf_filter_cfg(struct qed_hwfn *p_hwfn,
+		     struct qed_ntuple_filter_params *params);
+
 /**
  * @brief qed_vf_pf_int_cleanup - clean the SB of the VF
  *
@@ -2171,7 +2198,7 @@ u32 qed_vf_hw_bar_size(struct qed_hwfn *p_hwfn, enum BAR_ID bar_id);
  *
  * @param p_mac - mac address to be updated in bulletin board
  */
-int qed_vf_pf_bulletin_update_mac(struct qed_hwfn *p_hwfn, u8 * p_mac);
+int qed_vf_pf_bulletin_update_mac(struct qed_hwfn *p_hwfn, const u8 * p_mac);
 /**
  * @brief - qed_vf_pf_update_mtu Update MTU for VF.
  *          vport should not be active before issuing this function.
@@ -2319,6 +2346,8 @@ static inline int qed_vf_pf_rxq_start(struct qed_hwfn __maybe_unused * p_hwfn,
 static inline int qed_vf_pf_txq_start(struct qed_hwfn __maybe_unused * p_hwfn,
 				      struct qed_queue_cid __maybe_unused
 				      * p_cid,
+				      u8 __maybe_unused
+				      tc,
 				      dma_addr_t __maybe_unused
 				      pbl_addr,
 				      u16 __maybe_unused
@@ -2454,7 +2483,7 @@ static inline void qed_vf_set_vf_start_tunn_update_param(struct qed_tunnel_info
 
 static inline int qed_vf_pf_bulletin_update_mac(struct qed_hwfn __maybe_unused *
 						p_hwfn,
-						u8 __maybe_unused * p_mac)
+						const u8 __maybe_unused * p_mac)
 {
 	return -EINVAL;
 }
@@ -2465,6 +2494,13 @@ static inline int qed_vf_pf_update_mtu(struct qed_hwfn __maybe_unused * p_hwfn,
 	return -EINVAL;
 }
 
+static inline int qed_vf_pf_filter_cfg(struct qed_hwfn __maybe_unused * p_hwfn,
+				       struct qed_ntuple_filter_params
+				       __maybe_unused * params)
+{
+	return -EINVAL;
+}
+
 static inline int qed_vf_pf_establish_ll2_conn(struct qed_hwfn __maybe_unused *
 					       p_hwfn,
 					       u8 __maybe_unused
